%ADXL 335B (Analogovy)
clear all
clc
%vytvoreni objektu pro Arduino
a = arduino('COM5','Uno');

%vzorkovani
dt=0.2;
cas=0;
i=1;
%doba trvani
cas_konec=100;

%klidova hodnota napeti (pri zrychleni = 0 g), (napajeci napeti)/2
%viz datasheet (strana 12)
g_klid = (3.3/2);

%zmena vystupniho napeti pro 1 g, vypoctena pomoci linearni interpolace, 
%viz datasheet (strana 12)
g_napeti = (((0.36-0.195)/(3.6-2))*(3.3-2)) + 0.195;

%nekonecna smycka
while 1
    i = i + 1;
    t(i) = cas;
    cas = cas + dt;
    
    %Nacitani analogovych vystupu senzoru
    aX=readVoltage(a, 'A0');
    aY=readVoltage(a, 'A1');
    aZ=readVoltage(a, 'A2');
    
    %prepocet napeti na velikost zrychleni v jednotkach g
    X(i)=(aX - g_klid) / g_napeti;
    Y(i)=(aY - g_klid) / g_napeti;
    Z(i)=(aZ - g_klid) / g_napeti;
        
    %vykresleni polohy
    
    %souradnice pocatku
    POCATEK = zeros(3,3);
    %hodnoty slozek zrychleni (nacteny ze senzoru)
    ZRYCHLENI = [-X(i) -Y(i) -Z(i)];
    %vypis hodnot zrychleni na konzoli
    fprintf('x=%5.2f y=%5.2f z=%5.2f - cas:%5.1f\n',aX,aY,aZ,cas)
    %vytvoreni matice se slozkami zrychleni
    SLOZKY_ZRYCHLENI = [ZRYCHLENI(1)    0               0;      %x-ova slozka
                        0               ZRYCHLENI(2)    0;      %y-ova slozka
                        0               0               ZRYCHLENI(3)];  %z-ova slozka
    
    %vykresleni jednotlivych slozek zrychleni
    figure(1) 
    %quiver3(..) vykresluje 3D sipky od pocatku do konce
    quiver3(POCATEK(:,1), POCATEK(:,2), POCATEK(:,3), SLOZKY_ZRYCHLENI(:,1), SLOZKY_ZRYCHLENI(:,2), SLOZKY_ZRYCHLENI(:,3))
    axis equal
    axis([-1 1 -1 1 -1 1])
    xlabel('x'); 
    ylabel('y'); 
    zlabel('z');
    title('Slozky zrychleni v osach')
    
    %vykresleni souctu slozek zrychleni (normala plochy)
    figure(2) 
    %funkce arrow3() slouzi k vykresleni sipky v prostoru, jde o funkci
    %stazenou z mathworks.com
    arrow3(POCATEK(1, :), ZRYCHLENI)
    axis equal
    axis([-1 1 -1 1 -1 1])
    xlabel('x'); 
    ylabel('y'); 
    zlabel('z');
    grid on
    title('Soucet zrychleni')
    
    %vykresleni ctverce
    %pocatecni souradnice vrcholu ctverce
    p1 = [-2  -2 0];
    p2 = [ 2  -2 0];
    p3 = [ 2   2 0];
    p4 = [-2   2 0]; 
    %zapis souradnic do vektoru
    x = [p1(1) p2(1) p3(1) p4(1)];
    y = [p1(2) p2(2) p3(2) p4(2)];
    z = [p1(3) p2(3) p3(3) p4(3)];
    %matice souradnic vrcholu ctverce
    Souradnice = [x; y; z];
    %vektor [0 0 1] je vchoz normla ctverce, spocitam tedy nutnou rotaci
    %mezi vektorem zrychlen a vektorem [0 0 1] a aplikuji ji na ctverec
    %vytvorim matici rotace
    rotace = vrrotvec2mat(vrrotvec([0 0 1], ZRYCHLENI));    
    %rotace ctverce
    Souradnice = rotace * Souradnice; 
    figure(3)    
    %vykresleni ctverce
    senzor = fill3(Souradnice(1,:),Souradnice(2,:),Souradnice(3,:), 2);         
    xlabel('x'); 
    ylabel('y'); 
    zlabel('z');
    axis([-4 4 -4 4 -4 4])              %rozsah os
    grid on
    title('Poloha senzoru')
    drawnow
        
    %ukonceni vizualizace v pozadovany cas
    if cas>=cas_konec
        break
    end
    pause(dt)
end