%MPU 6050 (Digitalni)
clear all
clc
%vytvoreni objektu pro Arduino + nastaveni sbernice I2C
a = arduino('COM5', 'Uno', 'Libraries', 'I2C'); 

%objekt odkazujici na I2C sbernici
addrs = scanI2CBus(a);

%odkaz na prvni zarizeni na I2C
dev=i2cdev(a, addrs{1}) 

%nastaveni mericiho rozsahu na +- 2g - viz datasheet
writeRegister(dev, 28, bi2de([0 0 0 0 0 0 0 0 ],'left-msb'), 'uint8');

%proudit - opakovane mereni
writeRegister(dev, 107,bi2de([0 0 0 0 0 0 0 0 ],'left-msb'), 'uint8');

cas=0;
%vzorkovani
dT=0.05;

for i=1:100
    %Nacitani hodnot zrychleni pro jednotlive osy, cteni z prislusnych registru
    %viz datasheet
    %osa x:
    acc(1,1) = readRegister(dev, 59, 'int8');
    acc(1,2) = readRegister(dev, 60, 'int8');
    %osa y:
    acc(2,1) = readRegister(dev, 61, 'int8');
    acc(2,2) = readRegister(dev, 62, 'int8');
    %osa z:
    acc(3,1) = readRegister(dev, 63, 'int8');
    acc(3,2) = readRegister(dev, 64, 'int8');

    %Soucet dvou osmibitovych cisel - tim ziskame zrychleni v dane ose, deleni
    %cislem 16384 odpovida zvolenemu mezicimu rozsahu (viz datasheet), hodnoty
    %pro osy jsou ulozeny do matice
    ACC(:,i)=(double(acc(:,1))*256 + double(acc(:,2)))/16384; 

    %vykresleni polohy
    
    %souradnice pocatku
    POCATEK = zeros(3,3);
    %hodnoty slozek zrychleni (nacteny ze senzoru) [x, y, z]
    ZRYCHLENI = [ACC(1,i) ACC(2,i) -ACC(3,i)];
    %vypis hodnot zrychleni na konzoli
    fprintf('x=%5.2f y=%5.2f z=%5.2f - cas:%5.1f\n',ZRYCHLENI(1),ZRYCHLENI(2),ZRYCHLENI(3),cas)
    %vytvoreni matice se slozkami zrychleni
    SLOZKY_ZRYCHLENI = [ZRYCHLENI(1)    0               0;      %x-ova slozka
                        0               ZRYCHLENI(2)    0;      %y-ova slozka
                        0               0               ZRYCHLENI(3)];  %z-ova slozka
                    
    %vykresleni jednotlivych slozek zrychleni
    figure(1) 
    %quiver3(..) vykresluje 3D sipky od pocatku do konce
    quiver3(POCATEK(:,1), POCATEK(:,2), POCATEK(:,3), SLOZKY_ZRYCHLENI(:,1), SLOZKY_ZRYCHLENI(:,2), SLOZKY_ZRYCHLENI(:,3))
    axis equal
    axis([-1 1 -1 1 -1 1])
    xlabel('x'); 
    ylabel('y'); 
    zlabel('z');
    title('Slozky zrychleni')
    
    %vykresleni souctu slozek zrychleni (normala plochy)
    figure(2) 
    %funkce arrow3() slouzi k vykresleni sipky v prostoru, jde o funkci
    %stazenou z mathworks.com
    arrow3(POCATEK(1, :), ZRYCHLENI)
    axis equal
    axis([-1 1 -1 1 -1 1])
    xlabel('x'); 
    ylabel('y'); 
    zlabel('z');
    grid on
    title('Soucet zrychleni')
    
    %vykresleni ctverce
    %pocatecni souradnice vrcholu ctverce
    p1 = [-2  -2 0];
    p2 = [ 2  -2 0];
    p3 = [ 2   2 0];
    p4 = [-2   2 0]; 
    %zapis souradnic do vektoru
    x = [p1(1) p2(1) p3(1) p4(1)];
    y = [p1(2) p2(2) p3(2) p4(2)];
    z = [p1(3) p2(3) p3(3) p4(3)];
    %matice souradnic vrcholu ctverce
    Souradnice = [x; y; z];
    %vektor [0 0 1] je vchoz normla ctverce, spocitam tedy nutnou rotaci
    %mezi vektorem zrychlen a vektorem [0 0 1] a aplikuji ji na ctverec
    %vytvorim matici rotace
    rotace = vrrotvec2mat(vrrotvec([0 0 1], ZRYCHLENI));    
    %rotace ctverce
    Souradnice = rotace * Souradnice;
    figure(3)   
    %vykresleni ctverce
    senzor = fill3(Souradnice(1,:),Souradnice(2,:),Souradnice(3,:), 2);         
    xlabel('x'); 
    ylabel('y'); 
    zlabel('z');
    axis([-4 4 -4 4 -4 4]) 
    grid on
    title('Poloha senzoru')
    drawnow
        
    cas = cas + dT;
    pause(dT);
end